/****************************************************************************** 
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2008. All Rights Reserved.
 *
 * U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 ******************************************************************************/
dojo.provide("com.ibm.team.repository.web.ui.internal.ExternalTeamServerLoginDialog"); //$NON-NLS-1$

dojo.require("dijit._Widget"); //$NON-NLS-1$
dojo.require("dijit._Templated"); //$NON-NLS-1$

dojo.require("dojox.data.dom"); //$NON-NLS-1$

dojo.require("net.jazz.ajax.ui.internal.Dialog"); //$NON-NLS-1$

dojo.require("com.ibm.team.repository.web.foundation.ProgressMonitor"); //$NON-NLS-1$

dojo.requireLocalization("com.ibm.team.repository.web.ui.internal", "ExternalTeamServerClientMessages"); //$NON-NLS-1$ //$NON-NLS-2$

(function(){

var ProgressMonitor				= com.ibm.team.repository.web.foundation.ProgressMonitor;

var bind= dojo.string.substitute;
var MSGS= dojo.i18n.getLocalization("com.ibm.team.repository.web.ui.internal", "ExternalTeamServerClientMessages"); //$NON-NLS-1$ //$NON-NLS-2$

dojo.declare("com.ibm.team.repository.web.ui.internal.ExternalTeamServerLoginDialog",  //$NON-NLS-1$
			 [dijit._Widget, dijit._Templated], {

	// ---- constructor mixed-in properties --------------------------------------------------------------------------------	

	_dialogRoot: null,

	_titleContainer: null,
	_subTitleContainer: null,
	_descriptionContainer: null,
	_useridNode: null,
	_passwordNode: null,
	_loginError: null,
	_progressSection: null,
	_progressBar: null,
	_progressMessage: null,
	_loginButton: null,
	_cancelButton: null,

	_progressBarWidget: null,
	
	_requestLabel: null,
	_serverDescription: null,
	_loginHandler: null,
	
	_labelUserId: MSGS.LOGINDIALOG_LABEL_USERID,
	_labelPassword: MSGS.LOGINDIALOG_LABEL_PASSWORD,
	_labelLogin: MSGS.LOGINDIALOG_BUTTON_LOGIN,
	_labelCancel: MSGS.LOGINDIALOG_BUTTON_CANCEL,
	
	constructor: function() {
	},

	// ---- api ------------------------------------------------------------------------------------------------------------

	// ---- events ---------------------------------------------------------------------------------------------------------	

	onLoginSuccess: function() {
	},

	onCancel: function() {
	},

	// ---- widget implementation ------------------------------------------------------------------------------------------

	templatePath: dojo.moduleUrl("com.ibm.team.repository.web", "ui/internal/templates/ExternalTeamServerLoginDialog.html"), //$NON-NLS-1$ //$NON-NLS-2$

	postCreate: function() {
		this._progressBarWidget= new dijit.ProgressBar({}, this._progressBar);
		this._progressBarWidget.update({indeterminate: true});

		this._dialog = new net.jazz.ajax.ui.internal.Dialog({
			contentNode: this._dialogRoot,
			primaryTitle: MSGS.LOGINDIALOG_DIALOGTITLE,
			width: "30em", //$NON-NLS-1$
			isClosable: true
		});
		
		dojox.data.dom.replaceChildren(this._titleContainer, document.createTextNode(MSGS.LOGINDIALOG_TITLE));
		dojox.data.dom.replaceChildren(this._subTitleContainer, document.createTextNode(bind(MSGS.LOGINDIALOG_SUBTITLE, [ this._serverDescription.getName() ])));
		dojox.data.dom.replaceChildren(this._descriptionContainer, document.createTextNode(bind(MSGS.LOGINDIALOG_DESCRIPTION, [ this._requestLabel, this._serverDescription.getName(), this._serverDescription.getUrl() ])));
		
		this.connect(this._loginButton, "onclick", "_onLogin"); //$NON-NLS-1$ //$NON-NLS-2$
		this.connect(this._cancelButton, "onclick", "_onCancel"); //$NON-NLS-1$ //$NON-NLS-2$
	},

	// ---- implementation -------------------------------------------------------------------------------------------------

	_runWithProgress: function(progressMonitoredCode) {
		var monitor= new ProgressMonitor(dojo.hitch(this, this._notifyBeginTask), dojo.hitch(this, this._notifyDone));
		this._progressSection.style.visibility= "visible"; //$NON-NLS-1$
		progressMonitoredCode(monitor);
	},

	_onLogin: function() {
		this._runWithProgress(dojo.hitch(this, function(monitor) {
			var userid= this._useridNode.value;
			var password= this._passwordNode.value;
			var serverDescription= this._serverDescription;
			
			this._loginHandler(userid, password, serverDescription, dojo.hitch(this, function(status) {
				monitor.done();
				if (status.isOK()) {
					try {
						this.onLoginSuccess();
					} finally {
						this._dialog.close();
					}
				} else {
					dojox.data.dom.replaceChildren(this._loginError, document.createTextNode(status.getMessage()));
					this._passwordNode.value= ""; //$NON-NLS-1$
				}
			}), monitor);
		}));
	},

	_onCancel: function() {
		try {
			this.onCancel();
		} finally {
			this._dialog.close();
		}
	},

	_notifyBeginTask: function(name) {
		dojox.data.dom.replaceChildren(this._progressMessage, document.createTextNode(name));
	},

	_notifyDone: function() {
		this._progressSection.style.visibility= "hidden"; //$NON-NLS-1$
		dojox.data.dom.removeChildren(this._progressMessage);
	},
	
	__sentinel: null // terminates this class definition
});

com.ibm.team.repository.web.ui.internal.ExternalTeamServerLoginDialog.create= function(requestLabel, serverDescription, loginHandler) {
	return new com.ibm.team.repository.web.ui.internal.ExternalTeamServerLoginDialog({
		_requestLabel: requestLabel,
		_serverDescription: serverDescription,
		_loginHandler: loginHandler
	}, null);
};

})();
