/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2005-2008. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
dojo.provide("com.ibm.team.repository.web.ui.internal.wizard.WizardReview");

dojo.require("dijit._Widget");
dojo.require("dijit._Templated");
dojo.require("com.ibm.team.repository.web.client.internal.ServerStatusClient");
dojo.require("com.ibm.team.repository.web.transport.ServiceResponseHandler");
dojo.require("com.ibm.team.repository.web.ui.internal.admin.util");
dojo.require("dojo.i18n");

dojo.requireLocalization("com.ibm.team.repository.web.ui.internal.wizard", "WizardReviewMessages");


(function() {

var ServerStatusClient = com.ibm.team.repository.web.client.internal.ServerStatusClient;
var ServiceResponseHandler = com.ibm.team.repository.web.transport.ServiceResponseHandler;
var arrayIndexWithPropertyOfObject = com.ibm.team.repository.web.ui.internal.admin.util.arrayIndexWithPropertyOfObject;
var DB_SERVICE_NAME = "com.ibm.team.repository.service.internal.rdb.ConnectionPoolService";
var DB_VENDOR_NAME = "db.vendor";
var EMAIL_SERVICE_NAME = "com.ibm.team.repository.service.internal.MailerService";
var EMAIL_ENABLED_NAME = "notification.mail.enabled";

var USER_REGISTRY_SERVICE_NAME = "com.ibm.team.repository.service.internal.userregistry.ExternalUserRegistryService";
var USER_REGISTRY_NAME = "user.registry.type";
var LDAP_SERVICE_NAME = "com.ibm.team.repository.service.internal.userregistry.ldap.LDAPUserRegistryProvider";


var CONFIG_SECTION_COUNT = 3;

var messages;

dojo.declare("com.ibm.team.repository.web.ui.internal.wizard.WizardReview",  [dijit._Widget, dijit._Templated], {

	templatePath: dojo.moduleUrl("com.ibm.team.repository.web", "ui/internal/wizard/templates/WizardReview.html"),

	constructor: function() {
		this.messages = messages = dojo.i18n.getLocalization("com.ibm.team.repository.web.ui.internal.wizard", "WizardReviewMessages");
		this.JAZZ_ROOT = net.jazz.ajax._contextRoot;
		var tutorialLink = "<a href='https://jazz.net/learn/LearnItem.jsp?href=content/docs/client-tour/index.html' target=\"_blank\">";
		this.tutorial = dojo.string.substitute(this.messages.tutorialLinkStatement, [tutorialLink, "</a>"]); 
	},

	postCreate: function() {
		com.ibm.team.repository.web.ui.internal.wizard.WizardReview.superclass.postCreate.apply(this);
		this._navigation = this.wizardNav; 
		this._navigation.enablePrevious(true);
		this._navigation._wizardFinishButton.disabled = false;
	},
	
	// INITAL LOAD METHODS
	_refresh: function () {
		dojox.data.dom.removeChildren(this._summaryList);
		this._showConnectionStatus("PROGRESS", messages.loading);
		this._statusReturnCount = 0;
		this._fetchSummary();
	},
	
	_fetchSummary: function () {
		// db config
		var srh= new ServiceResponseHandler(this, "_handleDbProps", "_handleServiceFetchError");
		ServerStatusClient.getServiceConfiguration(srh, DB_SERVICE_NAME);
		
		//email config
		var srh= new ServiceResponseHandler(this, "_handleEmailProps", "_handleServiceFetchError");
		ServerStatusClient.getServiceConfiguration(srh, EMAIL_SERVICE_NAME);
		
		// user registry config
		var srh= new ServiceResponseHandler(this, "_handleUserRegistryProps", "_handleServiceFetchError");
		ServerStatusClient.getServiceConfiguration(srh, USER_REGISTRY_SERVICE_NAME);
	},
	
	_incrementStatusReturnCount: function () {
		this._statusReturnCount++;
		if (this._statusReturnCount == CONFIG_SECTION_COUNT) {
			this._showConnectionStatus("OK", messages.success);
		}
	},
	
	_handleDbProps: function(serviceInfo) {
		this._incrementStatusReturnCount();
		var vendor = "DB";
		for (var i=0; i<serviceInfo.properties.length; i++) {
			if (serviceInfo.properties[i].name === DB_VENDOR_NAME) {
				vendorProp = serviceInfo.properties[i]
				vendor = vendorProp.overrideValue != null ? vendorProp.overrideValue : vendorProp.defaultValue;
			}	
		};
		//KMB TODO add call to server info for databaseAvailable
		var dbStatus = vendor + messages.connectionSuccess; 
		var dbSummary = this._createConfigSummaryItem(messages.databaseConfiguration, dbStatus, serviceInfo.properties)
		this._summaryList.appendChild(dbSummary);
	},
	
	_handleEmailProps: function(serviceInfo) {
		this._incrementStatusReturnCount();
		var emailDisabled = true;
		for (var i=0; i<serviceInfo.properties.length; i++) {
			if (serviceInfo.properties[i].name === EMAIL_ENABLED_NAME) {
				emailProp = serviceInfo.properties[i]
				emailSetting = emailProp.overrideValue != null ? emailProp.overrideValue : emailProp.defaultValue;
				if (emailSetting === "true") {
					emailDisabled = false;
				}
			}	
		};
		var summaryLabel = messages.enabled;
		if (emailDisabled) {
			serviceInfo.properties = null;
			summaryLabel = messages.disabled;
		}
		var emailSummary = this._createConfigSummaryItem(messages.emailNotification, summaryLabel, serviceInfo.properties)
		this._summaryList.appendChild(emailSummary);
	}, 
	
	_handleUserRegistryProps: function(serviceInfo) {
		var index = arrayIndexWithPropertyOfObject(serviceInfo.properties, "name", "user.registry.type");
		var registryType = serviceInfo.properties[index];
		var registryTypeValue = registryType.overrideValue != null ? registryType.overrideValue : registryType.defaultValue;

		if (registryTypeValue === "LDAP") {
			// ldap config
			var srh= new ServiceResponseHandler(this, "_handleLdapProps", "_handleServiceFetchError");
			ServerStatusClient.getServiceConfiguration(srh, LDAP_SERVICE_NAME);
			return;
		} 
		registryTypeLabel = messages[registryTypeValue];
		
		this._incrementStatusReturnCount();
		var userRegistrySummary = this._createConfigSummaryItem(messages.userRegistry, registryTypeLabel, null)
		this._summaryList.appendChild(userRegistrySummary);
	},
	
	_handleLdapProps: function(serviceInfo) {
		this._incrementStatusReturnCount();
		var userRegistrySummary = this._createConfigSummaryItem(messages.userRegistry, "LDAP", serviceInfo.properties)
		this._summaryList.appendChild(userRegistrySummary);
	}, 
	
	_createConfigSummaryItem: function(label, summary, props){
		if (props != null && props.length > 0) {
			var item = document.createElement("a");
			item.href="#";
		} else {
			var item = document.createElement("div");
		}
		item.className = "summaryItem";
		
		var labelDiv = document.createElement("div");
		labelDiv.className = "summaryItemLabel";
		labelDiv.innerHTML = label;
		item.appendChild(labelDiv);
		
		var statusDiv = document.createElement("div");
		statusDiv.className = "summaryItemStatus";
		statusDiv.innerHTML = summary;
		item.appendChild(statusDiv);
		
		var itemClear = document.createElement("div");
		itemClear.className = "summaryItemClear";
		
		if (props != null && props.length > 0) {
			var itemProperties = this._createDetailedProperties(props);
			itemProperties.className = "summaryItemProperties";
			itemClear.appendChild(itemProperties);
			item._detailedProps = itemProperties;
			item._detailedProps.style.display = "none";
			
			item.onclick = function(event){
				dojo.stopEvent(event);
				if (item._detailedProps.style.display === "none") {
					var newDisplay = "";
					dojo.addClass(item, "expanded");
				} else {
					var newDisplay = "none";
					dojo.removeClass(item, "expanded");
				}
				item._detailedProps.style.display = newDisplay;
			}
		} 
		item.appendChild(itemClear);
		
		return item;
	}, 
	
	_createDetailedProperties: function (properties) {
		var tableDiv = document.createElement("table");
		var tableBody = document.createElement("tbody");
		
		var header = document.createElement("tr");
		var head1cell = document.createElement("th");
		var head1 = document.createTextNode(messages.property);
		head1cell.appendChild(head1);
		header.appendChild(head1cell);
		var head1cel2 = document.createElement("th");
		var head2 = document.createTextNode(messages.current);
		head1cel2.appendChild(head2);
		header.appendChild(head1cel2);
		tableBody.appendChild(header);
				
		for (var i=0; i<properties.length; i++) {
			if (properties[i].overrideValue != null || properties[i].defaultValue != null) {
				var row = document.createElement("tr");
				var nameCell = document.createElement("td");
				nameCell.appendChild(document.createTextNode(properties[i].displayName));
				row.appendChild(nameCell);
				var value = properties[i].overrideValue != null ? properties[i].overrideValue : properties[i].defaultValue;
				if (properties[i].name === "db.jdbc.location"){
					// replace ; with <WBR> for wrapping
					value = value.replace(/;/g,";<wbr>");
					value = value.replace(/:/g,":<wbr>");
				}
				var valHTML = document.createElement("div");
				valHTML.innerHTML = value;
				var valueCell = document.createElement("td");
				valueCell.appendChild(valHTML);
				row.appendChild(valueCell);
				tableBody.appendChild(row);
			}	
		};	
		
		tableDiv.appendChild(tableBody);
		return tableDiv;
	},
	
	_showConnectionStatus: function(code, statusMsg) {
		dojox.data.dom.removeChildren(this._stepStatus);
		var status = document.createElement("div");
		status.className = "stepStatus " + code;
		
		var statusImage = document.createElement("div");
		statusImage.className = "statusImage " + code+ "img";
		status.appendChild(statusImage);
		
		var message = document.createElement("div");
		dojo.addClass(message,"statusMsg");
		message.innerHTML = statusMsg;
		status.appendChild(message);
		
		this._stepStatus.appendChild(status);
	}, 
	
	_handleServiceFetchError: function() {
		this._showConnectionStatus("ERROR", messages.configError);
	}
});

})();